import type { ThemeRegistrationAny } from 'shiki'
import type { SyntaxHighlighterProps } from 'src/types/shiki'
import React from 'react'
import { Platform, ScrollView, StyleSheet, Text, type TextStyle, View } from 'react-native'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'
import { convertShikiTheme } from '../../utils/style-transformer'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

const styles = StyleSheet.create({
  scrollView: {
    flex: 1,
    minHeight: 20,
  },
  contentContainer: {
    flexGrow: 1,
  },
  lineContainer: {
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
})

const SyntaxHighlighter: React.FC<SyntaxHighlighterProps> = ({ text, language, languageId, theme }) => {
  const themeStyles = React.useMemo(() => convertShikiTheme(theme as ThemeRegistrationAny), [theme])
  const { tokens } = useSyntaxHighlighter({
    text,
    language,
    languageId,
    theme,
  })

  const renderToken = (token: any, index: number, lineIndex: number) => {
    const tokenStyle: TextStyle = {
      ...themeStyles.token,
      fontFamily: monospaceFont,
      ...(token.color && { color: token.color }),
      ...(token.fontStyle === 'italic' && themeStyles.italic),
      ...(token.fontWeight === 'bold' && themeStyles.bold),
    }

    return (
      <Text key={`${lineIndex}-${index}`} style={tokenStyle}>
        {token.content.replace(/ /g, '\u00A0')}
      </Text>
    )
  }

  const renderLine = (line: any[], lineIndex: number) => (
    <View key={lineIndex} style={styles.lineContainer}>
      {line.map((token, tokenIndex) => renderToken(token, tokenIndex, lineIndex))}
    </View>
  )

  return (
    <ScrollView horizontal showsHorizontalScrollIndicator={Platform.OS !== 'web'} style={styles.scrollView}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((line, index) => renderLine(line, index))}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
